package com.fjhx.sale.controller.claim;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.common.entity.claim.dto.ClaimDto;
import com.fjhx.common.entity.claim.dto.ClaimSelectDto;
import com.fjhx.common.entity.claim.po.Claim;
import com.fjhx.common.entity.claim.vo.ClaimContractVo;
import com.fjhx.common.entity.claim.vo.ClaimVo;
import com.fjhx.sale.service.claim.ClaimService;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.domain.BaseSelectDto;
import com.ruoyi.common.enums.BusinessType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;


/**
 * <p>
 * 到账认领 前端控制器
 * </p>
 *
 * @author
 * @since 2023-04-18
 */
@RestController
@RequestMapping("/claim")
public class ClaimController {

    @Autowired
    private ClaimService claimService;

    /**
     * 到账认领分页
     */
    @PostMapping("/page")
    public Page<ClaimVo> page(@RequestBody ClaimSelectDto dto) {
        return claimService.getPage(dto);
    }

    /**
     * 到账认领明细
     */
    @PostMapping("/detail")
    public ClaimVo detail(@RequestBody BaseSelectDto dto) {
        return claimService.detail(dto.getId());
    }

    /**
     * 到账认领新增
     */
    @PostMapping("/add")
    public void add(@RequestBody Claim claim) {
        claimService.add(claim);
    }

    /**
     * 到账认领编辑
     */
    @PostMapping("/edit")
    public void edit(@RequestBody ClaimDto claimDto) {
        claimService.edit(claimDto);
    }

    /**
     * 到账认领删除
     */
    @Log(title = "回款登记", businessType = BusinessType.DELETE)
    @PostMapping("/delete")
    public void delete(@RequestBody BaseSelectDto dto) {
        claimService.delete(dto.getId());
    }

    /**
     * 到账认领明细
     */
    @GetMapping("/sumClaimMoney")
    public BigDecimal sumClaimMoney(@RequestParam("businessId") Long businessId) {
        return claimService.sumClaimMoney(businessId);
    }


    /**
     * 查询认领记录
     */
    @PostMapping("/claimRecord")
    public List<ClaimContractVo> claimRecord(@RequestBody Claim claim) {
        return claimService.claimRecord(claim.getBusinessId());
    }

    /**
     * 查询到账认领明细数据
     */
    @PostMapping("/claimDetail")
    public Map<String, Object> claimDetail(@RequestBody Claim claim) {
        return claimService.claimDetail(claim.getBusinessId());
    }

    /**
     * 回款登记
     */
    @Log(title = "回款登记", businessType = BusinessType.INSERT)
    @PostMapping("/addClaimInfo")
    public void addClaimInfo(@RequestBody ClaimDto dto) {
        claimService.addClaimInfo(dto);
    }

    /**
     * 回款登记 分页
     */
    @PostMapping("/claimInfoPage")
    public Page<ClaimVo> claimInfoPage(@RequestBody ClaimSelectDto dto) {
        return claimService.claimInfoPage(dto);
    }

    /**
     * 回款确认 分页
     */
    @PostMapping("/claimInfoConfirmPage")
    public Page<ClaimVo> claimInfoConfirmPage(@RequestBody ClaimSelectDto dto) {
        return claimService.claimInfoConfirmPage(dto);
    }

    /**
     * 回款确认
     */
    @Log(title = "回款确认", businessType = BusinessType.UPDATE)
    @PostMapping("/claimConfirm")
    void claimConfirm(@RequestBody ClaimDto dto) {
        claimService.claimConfirm(dto);
    }

}
