package com.fjhx.sale.flow;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.fjhx.common.enums.CodingRuleEnum;
import com.fjhx.common.enums.FlowStatusEnum1;
import com.fjhx.common.service.coding.CodingRuleService;
import com.fjhx.flow.core.FlowDelegate;
import com.fjhx.flow.enums.FlowStatusEnum;
import com.fjhx.oa.entity.seal.po.SealConfig;
import com.fjhx.oa.service.seal.SealConfigService;
import com.fjhx.sale.entity.contract.po.Contract;
import com.fjhx.sale.entity.seal.dto.SealUseDto;
import com.fjhx.sale.entity.seal.po.SealUse;
import com.fjhx.sale.service.contract.ContractService;
import com.fjhx.sale.service.seal.SealUseService;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.utils.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Component
public class SealUseFlow extends FlowDelegate {

    @Autowired
    private SealUseService sealUseService;
    @Autowired
    private CodingRuleService codingRuleService;
    @Autowired
    private SealConfigService sealConfigService;
    @Autowired
    private ContractService contractService;

    @Override
    public String getFlowKey() {
        return "seal_use_flow";
    }

    @Override
    public Long start(Long flowId, JSONObject submitData) {
        SealUseDto dto = submitData.toJavaObject(SealUseDto.class);
        dto.setFlowId(flowId);

        dto.setCode(codingRuleService.createCode(CodingRuleEnum.SEAL_USE.getKey(), null));
        dto.setStatus(FlowStatusEnum1.UNDER_REVIEW.getKey());

        sealUseService.addOrEdit(dto);

        return dto.getId();
    }

    @Override
    public void end(Long flowId, Long businessId, JSONObject submitData) {
        SealUse byId = sealUseService.getById(businessId);
        sealUseService.update(q -> q
                .eq(SealUse::getId, businessId)
                .set(SealUse::getStatus, FlowStatusEnum1.PASS.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
        if (ObjectUtil.isNotEmpty(byId.getContractId())) {
            //刷新合同用印申请id
            contractService.update(q -> q.eq(Contract::getId, byId.getContractId()).set(Contract::getSealUseId, businessId));
        }
    }

    @Override
    public void relaunch(Long flowId, Long businessId, FlowStatusEnum flowStatus, JSONObject submitData) {
        start(flowId, submitData);
    }

    @Override
    public void reject(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        sealUseService.update(q -> q
                .eq(SealUse::getId, businessId)
                .set(SealUse::getStatus, FlowStatusEnum1.REJECT.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    @Override
    public void cancellation(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        sealUseService.cancellation(businessId);
    }

    @Override
    public List<Long> getHandlingUserList(Long flowId, JSONObject submitData) {
        SealUseDto dto = submitData.toJavaObject(SealUseDto.class);
        List<Long> sealIds = Arrays.stream(dto.getSealIds().split(",")).distinct().map(Long::valueOf).collect(Collectors.toList());
        //返回印章管理人列表
        List<Long> custodyUserId = sealConfigService.listObject(SealConfig::getCustodyUserId, q -> q.in(SealConfig::getId, sealIds));
        return custodyUserId.stream().distinct().collect(Collectors.toList());
    }
}
